<?php

/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * 表示可能なID(room_id,page_id,block_id)のリストを取得するFilter
 * TARGET_USER_ID or TARGET_LOGIN_ID or TARGET_MODULE_ID or TARGET_ROOM_ID
 * セットパラメータ名称 = "ALLOW_PAGE_ID" or "ALLOW_ROOM_ID" or "ALLOW_BLOCK_ID" 
 * or "ALLOW_BLOCK_ARR" or "ALLOW_PAGE_ARR"($pages[$thread_num][$parent_id][$display_sequence])
 * or "ALLOW_ROOM_ARR"($pages[$thread_num][$parent_id][$display_sequence])
 * 
 * @package     NetCommons.filter
 * @author      Noriko Arai,Ryuji Masukawa
 * @copyright   2006-2007 NetCommons Project
 * @license     http://www.netcommons.org/license.txt  NetCommons License
 * @project     NetCommons Project, supported by National Institute of Informatics
 * @access      public
 */
class Filter_AllowIdList extends Filter
{
    var $_classname = "Filter_AllowIdList";

    var $_container;
    var $_log;
    var $_actionChain;
    var $_filterChain;
    var $_pagesView;
    var $_blocksView;
    var $_session;
    var $_request;
    
    var $target_room_id = null;
    var $target_module_id = null;
    var $target_user_id = null;
    
    var $instance = null;
    //var $allow_room_id = null;
    //var $allow_page_id = null;
    //var $allow_block_id = null;
    //var $allow_page_arr = null;	//$pages[$thread_num][$parent_id][$display_sequence]毎
    //var $allow_block_arr = null;	//blocks[module_id]毎
    	
    
    /**
     * コンストラクター
     *
     * @access  public
     */
    function Filter_AllowIdList()
    {
        parent::Filter();
        //static $instance;
        
        //if ($instance === NULL) {
        //	$instance = new Filter_AllowIdList_Instance();
        //}
        //$this->instance =& $instance;
    }
    
    /**
     * コンストラクター
     *
     * @access  public
     */
    function setInstance($instance_key)
    {
    	if($this->instance !== null) {
    		return;	
    	}
        static $instance;
        if($instance === null) {
        	$instance = array();
        }
        if (!isset($instance[$instance_key])) {
        	$instance[$instance_key] = new Filter_AllowIdList_Instance();
        }
        $this->instance =& $instance[$instance_key];
    }

    /**
     * 表示可能なID取得処理を実行
     *
     * @access  public
     */
    function execute()
    {
        $this->_container   =& DIContainerFactory::getContainer();
        $this->_log         =& LogFactory::getLog();
        $this->_filterChain =& $this->_container->getComponent("FilterChain");
        //$this->_actionChain =& $this->_container->getComponent("ActionChain");
        $this->_pagesView   =& $this->_container->getComponent("pagesView");
        $this->_blocksView  =& $this->_container->getComponent("blocksView");
        $this->_session     =& $this->_container->getComponent("Session");
        $this->_request     =& $this->_container->getComponent("Request");

        $this->_log->trace("{$this->_classname}の前処理が実行されました", "{$this->_classname}#execute");
        $this->_preFilter();
        
        $this->_filterChain->execute();

        $this->_log->trace("{$this->_classname}の後処理が実行されました", "{$this->_classname}#execute");
        $this->_postFilter();
    }
    
    /**
     * プリフィルタ
     *
     * @access  private
     */
    function _preFilter()
    {
    	$set_arr = array();
    	//$action =& $this->_actionChain->getCurAction();
        $attributes = $this->getAttributes();
        $instance_key = "|";
        foreach ($attributes as $key => $value) {
        	$$key = array();
        	switch ($value) {
        		case "TARGET_SESSION_ROOM_ID":
        			//ターゲットルームID指定あり
        			$room_id = $this->_session->getParameter($key);
        			if(isset($room_id) && $room_id != "") {
        				$this->target_room_id = $room_id;
        				$instance_key .= "room_id=".$room_id."|";
        			}
        			break;
        		case "TARGET_ROOM_ID":
        			//ターゲットルームID指定あり
        			$room_id = $this->_request->getParameter($key);
        			if(isset($room_id) && $room_id != "") {
        				$this->target_room_id = $room_id;
        				$instance_key .= "room_id=".$room_id."|";
        			}
        			break;
        		case "TARGET_MODULE_ID":
        			//ターゲットモジュールID指定あり
        			$module_id = $this->_request->getParameter($key);
        			if(isset($module_id) && $module_id != "") {
        				$this->target_module_id = $module_id;
        				$instance_key .= "module_id=".$module_id."|";
        			}
        			break;
        		case "TARGET_LOGIN_ID":
        			//ターゲットログインID指定あり
        			$user_id = $this->_session->getParameter("_user_id");
        			if($user_id != "0") {
        				$this->target_user_id = $user_id;
        				$instance_key .= "user_id=".$user_id."|";
        			}
        			break;        	
				case "TARGET_USER_ID":
        			//ターゲット会員ID指定あり
        			$user_id = $this->_request->getParameter($key);
        			if($user_id != "0") {
        				$this->target_user_id = $user_id;
        				$instance_key .= "user_id=".$user_id."|";
        			}
        			break;        			
        		case "ALLOW_ROOM_ID":
        		case "ALLOW_PAGE_ID":
        		case "ALLOW_PAGE_ARR":
        		case "ALLOW_ROOM_ARR":
        		case "ALLOW_ROOM_ARR_FLAT":
        		case "ALLOW_PAGE_ARR_FLAT":
        			$this->setInstance($instance_key);
        			if($this->instance->allow_page_arr == null) {
        				$this->setShowPagesList();
        			}
        			
        			if($value == "ALLOW_ROOM_ID") {
        				$set_arr[$key] =& $this->instance->allow_room_id;
        			} else if ($value == "ALLOW_PAGE_ID") {
        				$set_arr[$key] =& $this->instance->allow_page_id;
        			} else if ($value == "ALLOW_ROOM_ARR_FLAT") {
        				$set_arr[$key] =& $this->instance->allow_room_arr_flat;
        			} else if ($value == "ALLOW_PAGE_ARR_FLAT") {
        				$set_arr[$key] =& $this->instance->allow_page_arr_flat;
        			} else if ($value == "ALLOW_ROOM_ARR") {
        				$set_arr[$key] =& $this->instance->allow_room_arr;
        			} else {
        				$set_arr[$key] =& $this->instance->allow_page_arr;
        			}
        			break;
        		case "ALLOW_BLOCK_ID":
        		case "ALLOW_BLOCK_ARR":
        			$this->setInstance($instance_key);
        			if($this->instance->allow_page_id == null) {
        				$this->setShowPagesList();
        			}
        			if($this->instance->allow_page_id != null) {
        				if( $this->target_module_id != null) {
        					$blocks_where_params = array("module_id" => $this->target_module_id);
        				} else {
        					$blocks_where_params = null;
        				}
        				$ret = $this->_blocksView->getBlockByPageId($this->instance->allow_page_id, $blocks_where_params, array($this, '_showblocks_fetchcallback'));
        				if($ret !== false) {
        					$this->instance->allow_block_id =& $ret[0];
							$this->instance->allow_block_arr =& $ret[1];
        				}
        			}
        			if($value == "ALLOW_BLOCK_ID") {
        				$set_arr[$key] =& $this->instance->allow_block_id;
        			} else {
        				$set_arr[$key] =& $this->instance->allow_block_arr;
        			}
        			break;
        	}
        	
        }
        if(isset($set_arr) && count($set_arr) > 0) {
        	foreach($set_arr as $key => $value) {
        		$this->_request->setParameter($key, $value);
        	}
        	//BeanUtils::setAttributes($action, $set_arr, true);
        }
    }

    /**
     * ポストフィルタ
     *
     * @access  private
     */
    function _postFilter()
    {
        // 何もしません。     
    }
    
    
	/**
	 *
	 * 表示可能ページ取得
	 *
	 */
	function setShowPagesList() {
		$order_params = array(
			"space_type" => "ASC",
			"private_flag" => "ASC",
			"thread_num" => "ASC",
			"display_sequence" => "ASC",
			"default_entry_flag" => "ASC"
		);
		if($this->target_user_id != null && $this->target_user_id != "0") {
			$where_params = array("user_id" => $this->target_user_id);
		} else {
			$where_params = null;
		}
		
		$ret = $this->_pagesView->getShowPagesList($where_params, $order_params, 0, 0, array($this, '_showpages_fetchcallback'));
		if($ret === false) {
			return false;
		}
		$this->instance->allow_room_id =& $ret[0];
		$this->instance->allow_page_id =& $ret[1];
		//$this->instance->allow_page_name =& $ret[2];
		$this->instance->allow_page_arr =& $ret[2];
		$this->instance->allow_room_arr =& $ret[3];
		
		$this->instance->allow_page_arr_flat =& $ret[4];
		$this->instance->allow_room_arr_flat =& $ret[5];
		
		return true;
	}
	
	/**
	 * fetch時コールバックメソッド(pages)
	 * @param result adodb object
	 * @access	private
	 */
	function _showpages_fetchcallback($result, $func_param) {
		$getdata =& $this->_container->getComponent("GetData");
		$config =& $getdata->getParameter("config");
		
		$column_row = array();
		if($config[_PAGESTYLE_CONF_CATID]['column_space_type_use']['conf_value'] == _OFF) {
			// スペースタイプ毎に左右カラム、ヘッダーを分けない
			$headercolumn_page_id = $this->_session->getParameter("_headercolumn_page_id");
	    	$leftcolumn_page_id = $this->_session->getParameter("_leftcolumn_page_id");
	    	$rightcolumn_page_id = $this->_session->getParameter("_rightcolumn_page_id");
	    	
	    	$headercolumn_flag[0] = false;
			$leftcolumn_flag[0] = false;
			$rightcolumn_flag[0] = false;
		
		} else {
			$headercolumn_page_id_str = $config[_PAGESTYLE_CONF_CATID]['headercolumn_page_id']['conf_value'];
			$leftcolumn_page_id_str   = $config[_PAGESTYLE_CONF_CATID]['leftcolumn_page_id']['conf_value'];
			$rightcolumn_page_id_str  = $config[_PAGESTYLE_CONF_CATID]['rightcolumn_page_id']['conf_value'];
			
			$headercolumn_page_id_arr = explode("|",$headercolumn_page_id_str);
			$leftcolumn_page_id_arr   = explode("|",$leftcolumn_page_id_str);
			$rightcolumn_page_id_arr  = explode("|",$rightcolumn_page_id_str);
			
			$headercolumn_flag[0] = false;
			$leftcolumn_flag[0] = false;
			$rightcolumn_flag[0] = false;
			
			$headercolumn_flag[1] = false;
			$leftcolumn_flag[1] = false;
			$rightcolumn_flag[1] = false;
			
			$headercolumn_flag[2] = false;
			$leftcolumn_flag[2] = false;
			$rightcolumn_flag[2] = false;
		}
		$user_id = $func_param[0];
		$user_auth_id = $func_param[1];
		
		$room_id_ret = array();
		$page_id_ret = array();
		//$page_name_ret = array();
		$ret = array();
		$ret_room = array();
		
		$ret_flat = array();
		$ret_room_flat = array();
		while ($row = $result->fetchRow()) {
			
			if(isset($this->target_room_id)) {
				//ターゲットルームあり
				//TODO:グループスペース直下の検索には現状、対応していない
				//グループスペースならば、display_positionがグループスペースでプレイベートフラグが１のものを通す
				//処理を入れる必要があり。要修正。
				if ($this->target_room_id != $row['room_id'] && $this->target_room_id != $row['page_id']) {
					continue;
				}
			}
			//if($row['action_name'] != "") {
			/*
			if($row['url'] == "") {
				$row['url'] = BASE_URL;
			}
			if($row['private_flag'] && $row['thread_num'] == 0) {
				//プライベートスペースのTop
				//TODO:今後、それぞれのプライベートに飛ぶように修正しなければならない
				//BASE_URL INDEX_FILE_NAME ?action=pages_view_main&_private_flag=1等と入力するとプライベートスペースのTopへ遷移する等といった具合に修正
				//現状、未対応
				$row['link_url'] = $row['url'].INDEX_FILE_NAME."?action=".$row['action_name'].$row['parameters']."&page_id=".$row['page_id'];
			} else {
				$row['link_url'] = $row['url'].INDEX_FILE_NAME."?action=".$row['action_name'].$row['parameters']."&page_id=".$row['page_id'];
			}
			*/
			if($row['system_flag'] == _ON && $row['role_authority_name'] != null) {
				if(defined($row['role_authority_name'])) {
					$row['role_authority_name'] = constant($row['role_authority_name']);
				}
			}
				
			//設定されてなければ、configのdefault値をセット
			$row['leftcolumn_flag'] = isset($row['leftcolumn_flag']) ?
						 $row['leftcolumn_flag'] : $config[_PAGESTYLE_CONF_CATID]['leftcolumn_flag']['conf_value'];
			$row['rightcolumn_flag'] = isset($row['rightcolumn_flag']) ?
							 $row['rightcolumn_flag'] : $config[_PAGESTYLE_CONF_CATID]['rightcolumn_flag']['conf_value'];
			$row['header_flag'] = isset($row['header_flag']) ?
							 $row['header_flag'] : $config[_PAGESTYLE_CONF_CATID]['header_flag']['conf_value'];
			$row['footer_flag'] = isset($row['footer_flag']) ?
							 $row['footer_flag'] : $config[_PAGESTYLE_CONF_CATID]['footer_flag']['conf_value'];
			
			$page_id_ret[] = $row['page_id'];
			//$page_name_ret[$row['page_id']] = $row['page_name'];
			if($row['page_id'] == $row['room_id']) {
				if($row['default_entry_flag'] == _ON && isset($row['role_authority_id']) && $row['role_authority_id'] == _ROLE_AUTH_OTHER) {
					$row['authority_id'] = _AUTH_OTHER;
					$row['hierarchy'] = _HIERARCHY_OTHER;
					continue;
				} else if($row['default_entry_flag'] == _ON && $row['authority_id'] == null) {
					if($row['private_flag'] == _ON) {
						$_default_entry_auth_private = $this->_session->getParameter("_default_entry_auth_private");
						if(isset($_default_entry_auth_private)) {
							$row['authority_id'] = $_default_entry_auth_private;
							$row['hierarchy'] = $this->_session->getParameter("_default_entry_hierarchy_private");
						}
					} elseif($row['space_type'] == _SPACE_TYPE_PUBLIC) {
						$_default_entry_auth_public = $this->_session->getParameter("_default_entry_auth_public");
						if(isset($_default_entry_auth_public)) {
							$row['authority_id'] = $_default_entry_auth_public;
							$row['hierarchy'] = $this->_session->getParameter("_default_entry_hierarchy_public");
						}
					} else {
						$_default_entry_auth_group = $this->_session->getParameter("_default_entry_auth_group");
						if(isset($_default_entry_auth_group)) {
							$row['authority_id'] = $_default_entry_auth_group;
							$row['hierarchy'] = $this->_session->getParameter("_default_entry_hierarchy_group");
						}
					}
					
					if($row['authority_id'] == null) {
						$row['authority_id'] = _AUTH_OTHER;
						$row['hierarchy'] = _HIERARCHY_OTHER;
					} else if($user_auth_id == _AUTH_GUEST && $row['authority_id'] == _AUTH_GENERAL) {
						$row['authority_id'] = _AUTH_GUEST;
					}
					
					// ゲスト　あるいは、一般で固定値
					if($row['authority_id'] == _AUTH_OTHER) {
						$row['role_authority_id'] = _ROLE_AUTH_OTHER;
						$row['role_authority_name'] = "";
					} else if($row['authority_id'] == _AUTH_GENERAL) {
						$row['role_authority_id'] = _ROLE_AUTH_GENERAL;
						$row['role_authority_name'] = _AUTH_GENERAL_NAME;
					} else {
						$row['role_authority_id'] = _ROLE_AUTH_GUEST;
						$row['role_authority_name'] = _AUTH_GUEST_NAME;
					}
				}
			}
			$ret[intval($row['thread_num'])][intval($row['parent_id'])][intval($row['display_sequence'])] = $row;
			$ret_flat[intval($row['page_id'])] =& $ret[intval($row['thread_num'])][intval($row['parent_id'])][intval($row['display_sequence'])];
			if($row['page_id'] == $row['room_id']) {
				//ルーム
				$room_id_ret[] = $row['page_id'];
				$ret_room[intval($row['thread_num'])][intval($row['parent_id'])][intval($row['display_sequence'])] =& $ret[intval($row['thread_num'])][intval($row['parent_id'])][intval($row['display_sequence'])];
				$ret_room_flat[intval($row['page_id'])] =& $ret[intval($row['thread_num'])][intval($row['parent_id'])][intval($row['display_sequence'])];
			}
			if($row['display_sequence'] == 0) {
				// 左右カラム or ヘッダー
				$column_row[] = $row;
				continue;	
			}
			//
			// 左右カラム、ヘッダーが表示可能ページかどうか
			//
			if($config[_PAGESTYLE_CONF_CATID]['column_space_type_use']['conf_value'] == _OFF) {
				$col_num = 0;
			} else {
				if($row['space_type'] == _SPACE_TYPE_PUBLIC) {
					$col_num = 0;
				} else if($row['private_flag'] == _ON) {
					$col_num = 1;
				} else {
					$col_num = 2;
				}
			}
			if($row['header_flag'] == _ON) {
				$headercolumn_flag[$col_num] = true;
			}
			if($row['leftcolumn_flag'] == _ON) {
				$leftcolumn_flag[$col_num] = true;
			}
			if($row['rightcolumn_flag'] == _ON) {
				$rightcolumn_flag[$col_num] = true;
			}
		}
		for($i = 0; $i< count($column_row); $i++) {
			if($config[_PAGESTYLE_CONF_CATID]['column_space_type_use']['conf_value'] == _OFF) {
				$col_num = 0;
			} else {
				if($column_row[$i]['space_type'] == _SPACE_TYPE_PUBLIC) {
					$col_num = 0;
				} else if($column_row[$i]['private_flag'] == _ON) {
					$col_num = 1;
				} else {
					$col_num = 2;
				}
				$headercolumn_page_id = $headercolumn_page_id_arr[$col_num];
				$leftcolumn_page_id = $leftcolumn_page_id_arr[$col_num];
				$rightcolumn_page_id = $rightcolumn_page_id_arr[$col_num];
			}
			if(($column_row[$i]['page_id'] == $headercolumn_page_id && $headercolumn_flag[$col_num] == true) ||
				($column_row[$i]['page_id'] == $leftcolumn_page_id && $leftcolumn_flag[$col_num] == true) ||
				($column_row[$i]['page_id'] == $rightcolumn_page_id && $rightcolumn_flag[$col_num] == true)
				) {
				// 閲覧可能ページ
				$ret[intval($column_row[$i]['thread_num'])][intval($column_row[$i]['parent_id'])][intval($column_row[$i]['display_sequence'])] = $column_row[$i];
				$ret_flat[intval($column_row[$i]['page_id'])] =& $ret[intval($column_row[$i]['thread_num'])][intval($column_row[$i]['parent_id'])][intval($column_row[$i]['display_sequence'])];
				$page_id_ret[] = $column_row[$i]['page_id'];
				//$page_name_ret[$column_row[$i]['page_id']] = $column_row[$i]['page_name'];
				if($column_row[$i]['page_id'] == $column_row[$i]['room_id']) {
					//ルーム
					$room_id_ret[] = $column_row[$i]['page_id'];
					$ret_room[intval($column_row[$i]['thread_num'])][intval($column_row[$i]['parent_id'])][intval($column_row[$i]['display_sequence'])] =& $ret[intval($column_row[$i]['thread_num'])][intval($column_row[$i]['parent_id'])][intval($column_row[$i]['display_sequence'])];
					$ret_room_flat[intval($column_row[$i]['page_id'])] =& $ret[intval($column_row[$i]['thread_num'])][intval($column_row[$i]['parent_id'])][intval($column_row[$i]['display_sequence'])];
				}
			}
			
		}
		return array($room_id_ret, $page_id_ret, $ret, $ret_room, $ret_flat, $ret_room_flat);
	}
	
	/**
	 * fetch時コールバックメソッド(blocks)
	 * @param result adodb object
	 * @access	private
	 */
	function _showblocks_fetchcallback($result) {
		$block_id_ret = array();
		$ret = array();
		while ($row = $result->fetchRow()) {
			$ret[] = $row;
			$block_id_ret[] = $row['block_id'];
		}
		return array($block_id_ret, $ret);
	}
}
// インスタンス保持
class Filter_AllowIdList_Instance
{
	var $allow_room_id = null;
    var $allow_page_id = null;
    //var $allow_page_name = null;
    var $allow_block_id = null;
    var $allow_page_arr = null;	//$pages[$thread_num][$parent_id][$display_sequence]毎
    var $allow_room_arr = null;
    var $allow_page_arr_flat = null;
    var $allow_room_arr_flat = null;
    var $allow_block_arr = null;	//blocks[module_id]毎
}

?>