<?php
/* vim: set expandtab tabstop=4 shiftwidth=4 softtabstop=4: */

/**
 * 携帯取得コンポーネント
 *
 * @package	 NetCommons Components
 * @author	  Noriko Arai,Ryuji Masukawa
 * @copyright   2006-2007 NetCommons Project
 * @license	 http://www.netcommons.org/license.txt  NetCommons License
 * @project	 NetCommons Project, supported by National Institute of Informatics
 * @access	  public
 */
class Mobile_View
{
	/**
	 * @var DIコンテナを保持
	 *
	 * @access	private
	 */
	var $_container = null;

	/**
	 * @var DBオブジェクトを保持
	 *
	 * @access	private
	 */
	var $_db = null;

	/**
	 * @var Sessionを保持
	 *
	 * @access	private
	 */
	var $_session = null;

	/**
	 * @var モジュール管理を保持
	 *
	 * @access	private
	 */
	var $_modulesView = null;

	/**
	 * @var モジュールを保持
	 *
	 * @access	private
	 */
	var $_modules = null;

	/**
	 * コンストラクター
	 *
	 * @access	public
	 */
	function Mobile_View() 
	{
		$this->_container =& DIContainerFactory::getContainer();
		$this->_db =& $this->_container->getComponent("DbObject");
		$this->_modulesView =& $this->_container->getComponent("modulesView");
		$this->_session =& $this->_container->getComponent("Session");
	}

	/**
	 * モジュールの取得
	 *
	 * @access	private
	 */
	function _callbackModules(&$recordSet) 
	{
		$result = array();
		while ($row = $recordSet->fetchRow()) {
			$pathList = explode("_", $row["mobile_action_name"]);
			$row["dir_name"] = $pathList[0];
			$row["module_name"] = $this->_modulesView->loadModuleName($row["dir_name"]);
			$result[$row["display_position"]][$row["module_id"]] = $row;
		}
		return $result;
	}
	/**
	 * モジュールの取得 ADD AllCreator 2009.02.04 
	 * 携帯のメニュー部分、モジュール単位ではなくページ単位＋モジュールに
	 * 2009.02.26:現在未使用
	 * @access  private
	 */
	function _callbackModulesByPage(&$recordSet)
	{
		$result = array();
		while ($row = $recordSet->fetchRow()) {
			$pathList = explode("_", $row["mobile_action_name"]);
			$row["dir_name"] = $pathList[0];
			$row["module_name"] = $this->_modulesView->loadModuleName($row["dir_name"]);
			$result[$row["display_position"]][$row["page_id"]][$row["module_id"]] = $row;
		}
		return $result;
	}
	/**
	 * 以下３つの関数：callbackRecursiveMenuByPage、_callbackMenuByPage、getNonDisplayPages
	 *
	 * ページタイトルの取得 ADD AllCreator 2009.02.07 
	 * 携帯のメニュー部分、ページ単位にする
	 * また、全頁に渡って非表示にされているページは明らかに隠しページなので、それは表示させない印をつけておく
	 *
	 * @access  private
	 */
	function callbackRecursiveMenuByPage( $res, $parent_id )
	{
		$result = array();
		foreach( $res as $r ) {
			if( $r["parent_id"] == $parent_id ) {
				$result[$r["page_id"]] = $r;
				if( $r["child_ct"] > 0 ) {
					// 連れてきた配列に対して、また子どもを連れてくる処理を行わせる
					$result2 = $this->callbackRecursiveMenuByPage( $res, $r["page_id"] );
					$result += $result2;
				}
			}
		}
		return $result;
	}
	/**
	 *
	 * Mod AllCreator 2009.05.05 
	 * 副問い合わせをSQLから削除した関係で、処理の一部を変更した
	 *
	 */
	function _callbackMenuByPage(&$recordSet)
	{
		$result = $recordSet;

		$result2 = array();
		while( $r = each( $result ) ) {
			$result2[$r["value"]["page_id"]] = $r['value'];
			if( $r["value"]["child_ct"] > 0 ) {
				// 子どもがいるなら子どもを全員連れてくる
				$result2 += $this->callbackRecursiveMenuByPage( $result, $r["value"]["page_id"] );
			}
		}
		//
		// check display or not display
		//

		// get thread nest depth
		$max_depth = $this->_db->maxExecute("pages", "thread_num" );
		if( $max_depth == 0 ) {
			// そのまま返す
			return $result2;
		}

		// get menu block count
		$sql = "SELECT COUNT(*) FROM {blocks} WHERE action_name like 'menu_%'";
		$menu_blk = $this->_db->execute( $sql, array(), NULL, NULL, false ); 
		if( $menu_blk == false ) {
			// そのまま返す
			return $result2;
		}

		$menu_blk_ct = $menu_blk[0][0];

		// get non-diplay pages list
		$non_display_pages = $this->getNonDisplayPages( $max_depth );

		foreach( $result2 as $page_id=>$r ) {
			if( isset( $non_display_pages[$page_id] ) ) {
				//if( $non_display_pages[$page_id] == $menu_blk_ct ) {
				if( $non_display_pages[$page_id] >= $menu_blk_ct ) {
					$r['visibility_flag'] = _OFF;
				}
				else {
					$r['visibility_flag'] = _ON;
				}
			}
			else {
				$r['visibility_flag'] = _ON;
			}
			$result2[$page_id] = $r;
		}
		
		return $result2;
	}
	/**
	 *
	 * Mod AllCreator 2009.05.05 
	 * 副問い合わせをSQLから削除するように立て直した
	 * 副問い合わせで実現していた部分はPHPコード処理に置き換えた
	 *
	 */
	function getNonDisplayPages( $max_depth )
	{
		$ret_ar = array();
		$work_ar = array();

		for( $i=1 ; $i<=$max_depth ; $i++ ) {

			$sql = "";

			$sql .= "SELECT ";

			for( $ii=1 ; $ii<=$i ; $ii++ ) {
				$sql .= "t{$ii}.page_id AS p{$ii}, t{$ii}.room_id AS r{$ii}, ";
			}

			for( $ii=$i+1 ; $ii<=$max_depth ; $ii++ ) {
				$sql .= "NULL AS p{$ii}, NULL AS r{$ii}, ";
			}

			$sql .= "t{$i}.page_id AS page_id ";
		
			$sql .= "FROM {pages} AS t1 ";

			for( $ii=2 ; $ii<=$i ; $ii++ ) {
				$parent_tbl_id = $ii -1;
				$sql .= "LEFT JOIN {pages} AS t{$ii} ON t{$ii}.parent_id = t{$parent_tbl_id}.page_id ";
			}
			$sql .= "WHERE t1.thread_num = 1 ";

			$tmp_ar = $this->_db->execute( $sql ); 

			foreach( $tmp_ar as $key=>$r ) {
				if( $r['page_id'] == NULL ) {
					for( $parent_page_i=$i ; $parent_page_i>0 ; $parent_page_i-- ) {
						$index_name = "p" . $parent_page_i;
						if( $r[ $index_name ] != NULL ) {
							$tmp_ar[$key]['page_id'] = $r[$index_name];
							break;
						}
					}
				}
			}
			$work_ar = array_merge( $work_ar,  $tmp_ar );
		}

		foreach( $work_ar as $r ) {
			$push_flg = true;
			foreach( $ret_ar as $diff_r ) {
				if( $r == $diff_r ) {
					$push_flg = false;
					break;
				}
			}
			if( $push_flg == true ) {
				$ret_ar[] = $r;
			}
		}

		$non_display_pages = array();
		$sql = 
			" ( SELECT temp_name, {menu_detail}.* FROM {menu_detail} INNER JOIN {blocks} ON {menu_detail}.block_id = {blocks}.block_id ) " ;
		$menus_ar = $this->_db->execute( $sql ); 
		if (empty($menus_ar)) {
			return $non_display_pages;
		}

		foreach( $menus_ar as $r ) {

			foreach( $ret_ar as $tmp ) {
				if (!isset($non_display_pages[ $tmp['page_id'] ])) {
					$non_display_pages[ $tmp['page_id'] ] = 0;
				}

				//
				// 全てのページ番号とルーム番号の組み合わせが異なることの確認
				// （これらが一緒＝カテゴリ）
				//
				$is_category = false;
				for( $i=$max_depth ; $i>0 ; $i-- ) {
					$page_no_index = "p" . $i;
					$room_no_index = "r" . $i;
					if( $tmp[$page_no_index] != null ) {
						if( $tmp[$page_no_index] == $tmp[$room_no_index] ) {
							$is_category = true;
							break;
						}
					}
				}
				//
				// メニュータイプにFLATの文字列があれば
				//
				if( strpos( $r['temp_name'] , 'flat' ) !== false && $is_category==false ) {
					if( $r['page_id'] == $tmp['page_id'] ) {
						$non_display_pages[ $tmp['page_id'] ] ++;
					}
				}
				else {
					$addtion_flg = false;
					for( $i=$max_depth ; $i>0 ; $i-- ) {
						$page_no_index = "p" . $i;
						if( $r['page_id'] == $tmp[$page_no_index] ) {
							$addtion_flg = true;
							break;
						}
					}
					if( $addtion_flg == true ) {
						$non_display_pages[ $tmp['page_id'] ] ++;
					}
				}
			}
		}

		return $non_display_pages;
	}

	/**
	 * ブロックタイトルの取得 add by AllCreator 2009.02.07 
	 * 携帯の第２メニュー部分、ブロック単位にする
	 *
	 * @access  private
	 */
	function _callbackBlockByPage(&$recordSet)
	{
		$result = array();
		while ($row = $recordSet->fetchRow()) {
			$pathList = explode("_", $row["mobile_action_name"]);
			$row["dir_name"] = $pathList[0];
			$row["module_name"] = $this->_modulesView->loadModuleName($row["dir_name"]);
			$result[$row["display_position"]][$row["block_id"]] = $row;
		}
		return $result;
	}
	/**
	 * モジュールの取得
	 *
	 * @access	public
	 */
	function getModules($module_id=null, $func=null) 
	{
		$actionChain =& $this->_container->getComponent("ActionChain");
		$actionName = $actionChain->getCurActionName();

		if ($this->_modules) {
			if (isset($module_id)) {
				if (isset($this->_modules[_DISPLAY_POSITION_CENTER][$module_id])) return $this->_modules[_DISPLAY_POSITION_CENTER][$module_id];
				if (isset($this->_modules[_DISPLAY_POSITION_HEADER][$module_id])) return $this->_modules[_DISPLAY_POSITION_HEADER][$module_id];
			} else {
				return $this->_modules;
			}
		}

		$sql = "SELECT {mobile_modules}.*" .
				" FROM {modules}" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				(preg_match("/^mobile_/", $actionName) ? "" : " WHERE {mobile_modules}.use_flag=". _ON) .
				" ORDER BY {mobile_modules}.display_position DESC, {mobile_modules}.display_sequence, {modules}.display_sequence";

		if (!isset($func)) {
			$func = array($this,"_callbackModules");
		}
		$this->_modules = $this->_db->execute($sql, null, null, null, true, $func);
		if ($this->_modules === false) {
		   	$this->_db->addError();
		   	return $this->_modules;
		}

		if (isset($module_id)) {
			if (isset($this->_modules[_DISPLAY_POSITION_CENTER][$module_id])) return $this->_modules[_DISPLAY_POSITION_CENTER][$module_id];
			if (isset($this->_modules[_DISPLAY_POSITION_HEADER][$module_id])) return $this->_modules[_DISPLAY_POSITION_HEADER][$module_id];
		} else {
			return $this->_modules;
		}
	}
	/**
	 * モジュールの取得
	 *
	 * @access	public
	 */
	function getModulesByRoom($room_id=null, $func=null) 
	{
		$actionChain =& $this->_container->getComponent("ActionChain");
		$actionName = $actionChain->getCurActionName();

		$sql = "SELECT DISTINCT {mobile_modules}.*" .
				" FROM {blocks}" .
				" INNER JOIN {pages} ON ({pages}.page_id = {blocks}.page_id)" .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id)" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				" WHERE 1=1" .
				(isset($room_id) ? " AND {pages}.room_id = ".intval($room_id) : "") .
				(preg_match("/^mobile_/", $actionName) ? "" : " AND {mobile_modules}.use_flag=". _ON) .
				" ORDER BY {mobile_modules}.display_position DESC, {mobile_modules}.display_sequence, {modules}.display_sequence";

		if (!isset($func)) {
			$func = array($this,"_callbackModules");
		}
		$modules = $this->_db->execute($sql, null, null, null, true, $func);
		if ($modules === false) {
		   	$this->_db->addError();
		   	return $modules;
		}
		if (isset($modules[_DISPLAY_POSITION_CENTER])) {
			return $modules[_DISPLAY_POSITION_CENTER];
		} else {
			return array();
		}
	}
	/**
	 * モジュールの取得 add by AllCreator 2009.02.04 
	 * 携帯のメニュー、ページ単位＋モジュール単位にする
	 * 2009.02.26 : 現在未使用
	 *
	 * @access  public
	 */
	function getModulesByRoomAndPage($room_id=null, $func=null)
	{
		$actionChain =& $this->_container->getComponent("ActionChain");
		$actionName = $actionChain->getCurActionName();

		$sql = "SELECT DISTINCT {mobile_modules}.*, {pages}.page_id, {pages}.page_name, {pages}.display_position, {pages}.display_sequence" .
				" FROM {blocks}" .
				" INNER JOIN {pages} ON ({pages}.page_id = {blocks}.page_id)" .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id)" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				" WHERE 1=1" .
				(isset($room_id) ? " AND {pages}.room_id = ".intval($room_id) : "") .
				(preg_match("/^mobile_/", $actionName) ? "" : " AND {mobile_modules}.use_flag=". _ON) .
				" ORDER BY {pages}.display_sequence, {mobile_modules}.display_position DESC, {mobile_modules}.display_sequence, {modules}.display_sequence";

		if (!isset($func)) {
			//$func = array($this,"_callbackModules");
			$func = array($this,"_callbackModulesByPage");
		}
		$modules = $this->_db->execute($sql, null, null, null, true, $func);
		if ($modules === false) {
			$this->_db->addError();
			return $modules;
		}
		if (isset($modules[_DISPLAY_POSITION_CENTER])) {
			return $modules[_DISPLAY_POSITION_CENTER];
		} else {
			return array();
		}
	}
	/**
	 * メニュー項目の取得 add by AllCreator 2009.02.04 
	 *					mod by AllCreator 2009.05.05 do not use sub query in SQL
	 *					mod by AllCreator 2009.05.06 do not use preg_match ここでmobile_で始まるアクションはこない
	 *												 (preg_match("/^mobile_/", $actionName) ? "" : " AND {mobile_modules}.use_flag=". _ON) .
	 * 携帯のメニュー画面用
	 * ページ単位でメニューにする
	 * メニューはPCでは、ページ＆ブロック単位で情報が管理されて、表示／非表示項目を決めている。
	 * 残念ながら携帯ではブロック単位でのメニュー表示が不可能なので、ブロック別に管理されているメニュー非表示項目を集約するしかない
	 * （少なくとも現状では）
	 * 基本設計：
	 * １．TOPのメニューで全てのページを集約したツリー型のメニューを表示する
	 * ２．ただし、全てのメニューブロックにわたって「非表示」が指定されているページについては「隠し」として認識し、このツリーメニューには含めない
	 * ３．～では表示、～では非表示、という部分制約のメニュー項目はTOPメニューに表示する。
	 *
	 * 問題点：携帯で表示できないモジュールだけで作成されているページの場合はどうするのだ？
	 * 　　　　現時点では「携帯対応」なのかどうなのかは考えないことにする。
	 *		 理想は「全てのモジュールが携帯表示に対応する」です。
	 *
	 * @access  public
	 */
	function getMenuItemsByRoomAndPage($room_id=null, $func=null)
	{
//		$actionChain =& $this->_container->getComponent("ActionChain");
//		$actionName = $actionChain->getCurActionName();
		//
		// メインの表示すべきであろうメニュー項目（ページ）の一覧を取得する
		//
		$sql = 
				" SELECT DISTINCT {pages}.parent_id, {pages}.page_id, {pages}.page_name, " .
				" {pages}.display_position, {pages}.display_sequence, {pages}.thread_num, " .
				" 0 as child_ct " .
				" FROM {blocks}" .
				" INNER JOIN {pages} ON ({pages}.page_id = {blocks}.page_id)" .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id)" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				" WHERE 1=1" .
				(isset($room_id) ? " AND {pages}.room_id = ".intval($room_id) : "") .
				" AND {mobile_modules}.use_flag=". _ON .
				" ORDER BY thread_num, display_sequence  ";
		$rec1 = $this->_db->execute($sql, null, null, null, true );
		if ($rec1 === false) {
			$this->_db->addError();
			return $rec1;
		}

		$result = array();
		foreach ($rec1 as $row ) {
			$result[$row["display_position"]][$row["page_id"]] = $row;
		}
		if (isset($result[_DISPLAY_POSITION_CENTER])) {
			$result = $result[_DISPLAY_POSITION_CENTER];
		} else {
			$result = array();
		}

		//
		// 先ほどのメニュー（ページ）一覧に、そのメニュー（ページ）に含まれるサブメニュー数情報を加える
		//
		$sql =
				" select {pages}.parent_id, count(*) as child_ct from {pages} " .
				" INNER JOIN {blocks} ON ({blocks}.page_id = {pages}.page_id)" .
				" INNER JOIN {mobile_modules} ON ({blocks}.module_id = {mobile_modules}.module_id)" .
				" WHERE 1=1" .
				" AND {mobile_modules}.use_flag=". _ON .
				(isset($room_id) ? " AND {pages}.room_id = ".intval($room_id) : "") .
				" group by {pages}.parent_id ";
		$rec2 = $this->_db->execute($sql, null, null, null, true );
		if ($rec2 === false) {
			$this->_db->addError();
			return $rec2;
		}
		foreach ( $rec2 as $row ) {
			if( isset( $result[ $row['parent_id'] ] ) ) {
				$result[ $row['parent_id'] ]['child_ct'] = $row['child_ct'];
			}
		}

		//
		// 先ほどのメニュー（ページ）一覧に、そのメニュー（ページ）に含まれるブロック数情報を加える
		//
		$sql =
				" select {blocks}.page_id, max({mobile_modules}.module_id) as module_id, max({blocks}.block_id) as block_id, count({blocks}.block_id) as block_ct " .
				" FROM {blocks} " .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id) " .
				" INNER JOIN {mobile_modules} ON ( {modules}.module_id = {mobile_modules}.module_id ) " .
				" GROUP BY {blocks}.page_id " ;
		$rec3 = $this->_db->execute($sql, null, null, null, true);
		if ($rec3 === false) {
			$this->_db->addError();
			return $rec3;
		}
		foreach ( $rec3 as $row ) {
			if( isset( $result[ $row['page_id'] ] ) ) {
				$result[ $row['page_id'] ]['block_id'] = $row['block_id'];
				$result[ $row['page_id'] ]['block_ct'] = $row['block_ct'];
			}
		}

		$modules = $this->_callbackMenuByPage( $result );

		if( is_array( $modules ) ) {
			return $modules;
		}
		else {
			return array();
		}
	}
	/**
	 * ブロック項目の取得 add by AllCreator 2009.02.07 
	 * mod by AllCreator 2009.05.06 cut [WHERE 1=1]
	 * mod by AllCreator 2009.05.06 cut (preg_match("/^mobile_/", $actionName)
	 * 携帯のメニュー画面用
	 * 指定されたページの中に存在するブロック一覧を取得する。
	 * タイトルのないブロックでかつ「おしらせ」モジュールの場合はそのまま中身を表示
	 * また、ここでブロックが１つしかないときは、すぐにそのモジュールのdetail
	 * 表示処理ページに行くのでこの関数は呼ばれない
	 *
	 * @access  public
	 */
	function getBlocksByPage($page_id=null, $func=null)
	{
//		$actionChain =& $this->_container->getComponent("ActionChain");
//		$actionName = $actionChain->getCurActionName();

		$sql = "SELECT {mobile_modules}.*, {blocks}.*, {announcement}.content " .
				" FROM {blocks}" .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id)" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				" LEFT JOIN {announcement} ON {blocks}.block_id = {announcement}.block_id" .
				" WHERE {blocks}.page_id = " . intval($page_id) .
				" AND {mobile_modules}.use_flag=". _ON .
				" ORDER BY {blocks}.col_num, {blocks}.row_num, {mobile_modules}.display_position DESC, {mobile_modules}.display_sequence";
		if (!isset($func)) {
			$func = array($this,"_callbackBlockByPage");
		}
		$modules = $this->_db->execute($sql, null, null, null, true, $func);
		if ($modules === false) {
			$this->_db->addError();
			return $modules;
		}
		if (isset($modules[_DISPLAY_POSITION_CENTER])) {
			return $modules[_DISPLAY_POSITION_CENTER];
		} else {
			return array();
		}
	}
	/**
	 * ブロックとページ項目の取得 add by AllCreator 2009.02.07 
	 * mod by AllCreator 2009.05.06 cut [WHERE 1=1]
	 * mod by AllCreator 2009.05.06 cut (preg_match("/^mobile_/", $actionName)
	 * 携帯のメニュー画面用
	 * 指定されたページの中に存在するブロックとページ一覧を取得する。
	 * 2009.02.26 : 現在未使用
	 * @access  public
	 */
	function getBlocksAndMenuItemByPage($page_id=null, $func=null)
	{
//		$actionChain =& $this->_container->getComponent("ActionChain");
//		$actionName = $actionChain->getCurActionName();

		$sql = "SELECT {mobile_modules}.*, {blocks}.*,  {pages}.page_id, {pages}.page_name, {pages}.display_position, {pages}.display_sequence " .
				" FROM {blocks}" .
				" INNER JOIN {modules} ON ({blocks}.module_id = {modules}.module_id)" .
				" INNER JOIN {mobile_modules} ON ({modules}.module_id = {mobile_modules}.module_id)" .
				" WHERE {blocks}.page_id = " . intval($page_id) .
				" AND {mobile_modules}.use_flag=". _ON .
				" ORDER BY {blocks}.col_num, {blocks}.row_num, {mobile_modules}.display_position DESC, {mobile_modules}.display_sequence";

		if (!isset($func)) {
			$func = array($this,"_callbackBlockByPage");
		}
		$modules = $this->_db->execute($sql, null, null, null, true, $func);
		if ($modules === false) {
			$this->_db->addError();
			return $modules;
		}
		if (isset($modules[_DISPLAY_POSITION_CENTER])) {
			return $modules[_DISPLAY_POSITION_CENTER];
		} else {
			return array();
		}
	}
	/**
	 * モジュールの取得
	 *
	 * @access	public
	 */
	function getCount($module_id) 
	{
		$count = $this->_db->countExecute("mobile_modules", array("module_id"=>$module_id));
		if ($count === false) {
			return false;
		}
		return $count;
	}
	/**
	 * 指定ブロックに該当するページの中にあるモバイルで表示できるブロック数の取得
	 * 2009.02.25 add by AllCreator
	 * 
	 * default return : 1;
	 *
	 * @access	public
	 */
	function getCountForBlockInPageWithBlock($block_id) 
	{
		$result = $this->_db->selectExecute( "blocks", array( "block_id"=>$block_id) );
		if ($result == false ) {
			return 1;
		}
		$page_id = $result[0]['page_id'];
		$ret = $this->getCountForBlockInPage($page_id);

		return $ret;
	}
	/**
	 * 指定ページの中にあるモバイルで表示できるブロック数の取得
	 * 2009.02.13 add by AllCreator
	 * 2009.05.05 mod by AllCreator
	 * 2009.05.06 mod by AllCreator cut [WHERE 1=1]
	 * 2009.05.06 mod by AllCreator cut (preg_match("/^mobile_/", $actionName)
	 *
	 * @access	public
	 */
	function getCountForBlockInPage($page_id) 
	{
//		$actionChain =& $this->_container->getComponent("ActionChain");
//		$actionName = $actionChain->getCurActionName();
		$sql = " SELECT module_id from {mobile_modules} where use_flag = " . _ON  ;
		$result = $this->_db->execute( $sql, null, null, null, true );
		if ($result === false) {
			$this->addError();
			return false;
		}
		$where_in_words = "";
		foreach( $result as $r ) {
			$where_in_words .= $r['module_id'] . ",";
		}
		$where_in_words = trim( $where_in_words, "," );

		$sql = " SELECT count(*) " .
				" FROM {blocks}" .
				" WHERE {blocks}.page_id = " . intval($page_id) .
				" AND {blocks}.module_id in ( $where_in_words )"  ;
		$result = $this->_db->execute($sql, null, null,null,false);
		if ($result === false) {
			$this->addError();
			return false;
		}
		return $result[0][0];
	}

	/**
	 * 端末IDの取得
	 *
	 * @access	public
	 */
	function getAutoLogin() 
	{
		$mobile_info = $this->_session->getParameter("_mobile_info");
		if ($mobile_info["autologin"] != _AUTOLOGIN_OK) { return false; }

		$maxlifetime = $this->_session->getParameter('_session_gc_maxlifetime');
		$time = timezone_date();
		$timestamp = mktime(intval(substr($time,8,2)),intval(substr($time,10,2))-$maxlifetime,intval(substr($time,12,2)),
							intval(substr($time,4,2)),intval(substr($time,6,2)),intval(substr($time,0,4)));
		$sql = "SELECT COUNT(*) FROM {mobile_users} WHERE tel_id = ? AND update_time >= ?";
		$result = $this->_db->execute($sql, array("tel_id"=>$mobile_info["tel_id"], "update_time"=>date("YmdHis", $timestamp)),null,null,false);
		if (intval($result[0][0]) == 0) { return false; }

		$result = $this->_db->selectExecute("mobile_users", array("tel_id"=>$mobile_info["tel_id"]));
		if ($result === false) { return false; }

		return $result[0];
	}
	/**
	 * ページ情報の取得
	 * 2009.02.04 add by AllCreator
	 * 2009.05.06 mod by AllCreator cut [WHERE 1=1]
	 *
	 * @access public
	 */
	function getPageName($page_id)
	{
		//$page_id = $this->_session->getParameter("page_id");
		//if( $page_id == '' ) {
		//  return '';
		//}
		$sql = "SELECT {pages}.page_name" .
				" FROM {pages}" .
				" WHERE {pages}.page_id = ".intval($page_id) ;
		$pages = $this->_db->execute($sql, null, null, null, true, null);
		if ($pages === false) {
			$this->_db->addError();
			return '';
		}
		if (isset($pages['page_name'])) {
			return $pages['page_name'];
		} else {
			return '';
		}
	}
}
?>