<?php
/**
 * モジュールアップデートクラス
 * 　　css_filesにカラム追加
 * 
 * @package     NetCommons.components
 * @author      Noriko Arai,Ryuji Masukawa
 * @copyright   2006-2007 NetCommons Project
 * @license     http://www.netcommons.org/license.txt  NetCommons License
 * @project     NetCommons Project, supported by National Institute of Informatics
 * @access      public
 */
class Module_Update extends Action
{	
	//使用コンポーネントを受け取るため
	var $db = null;
	
	function execute()
	{
		$adodb = $this->db->getAdoDbObject();
		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."css_files");
		if(!isset($metaColumns["SYSTEM_FLAG"])) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."css_files` 
						ADD `system_flag` TINYINT( 3 ) NOT NULL DEFAULT '0' AFTER `data` ,
						ADD `common_general_flag` TINYINT( 3 ) NOT NULL DEFAULT '0' AFTER `system_flag` ,
						ADD `common_admin_flag` TINYINT( 3 ) NOT NULL DEFAULT '0' AFTER `common_general_flag` ;";
			$result = $this->db->execute($sql);
			if($result === false) return false;
		}
		
		// index insert_user_id
		$sql = "SHOW INDEX FROM `".$this->db->getPrefix()."pages` ;";
		$results = $this->db->execute($sql);
		if($results === false) return false;
		$alter_table_flag = true;
		foreach($results as $result) {
			if(isset($result['Column_name']) && $result['Column_name'] == "insert_user_id" &&
				$result['Key_name'] == "insert_user_id") {
				$alter_table_flag = false;
			}
		}
		if($alter_table_flag) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."pages` 
						ADD INDEX ( `insert_user_id` ) ;";
			$result = $this->db->execute($sql);
			if($result === false) return false;
		}
		
		// index action_name
		$sql = "SHOW INDEX FROM `".$this->db->getPrefix()."modules` ;";
		$results = $this->db->execute($sql);
		if($results === false) return false;
		$alter_table_flag = true;
		foreach($results as $result) {
			if(isset($result['Column_name']) && $result['Column_name'] == "action_name" &&
				$result['Key_name'] == "action_name") {
				$alter_table_flag = false;
			}
		}
		if($alter_table_flag) {
			$sql = "ALTER TABLE `".$this->db->getPrefix()."modules` 
						ADD INDEX ( `action_name` ) ;";
			$result = $this->db->execute($sql);
			if($result === false) return false;
		}
		
		$max_id = $this->db->maxExecute("modules_seq_id", "id");
		if($max_id !== false && $max_id > 0) {
			$where_params = array("id !=".intval($max_id) => null);
			$this->db->deleteExecute("modules_seq_id", $where_params);
		}
		//
		// Group Left Column:pagesテーブル:page_id=10のdefault_entry_flag=2の値をdefault_entry_flag=1に更新
		// configテーブル:column_space_type_useが1の場合、影響あり。
		// （一般がグループルームへ遷移するとエラーとなった）
		//
		$pages = $this->db->selectExecute("pages", array("page_id"=> 10,"default_entry_flag" => 2));
		if($pages !== false && count($pages) > 0) {
			$this->db->updateExecute("pages", array("default_entry_flag" => _ON), array("page_id" => 10));
		}
		//
		// page_id=12のDefault Private RoomをDefault Roomに変更し、space_type=0,private_flag=0,action_name=''に変更
		//
		$pages = $this->db->selectExecute("pages", array("page_id"=> 12, 'private_flag'=> _ON));
		if($pages !== false && count($pages) > 0) {
			$this->db->updateExecute("pages", array("page_name" => 'Default Room', 'private_flag'=> _OFF, 'action_name'=> '', 'space_type'=> 0), array("page_id" => 12));
		}
		//
		// private_spaceのinsert_user_idを自分自身に更新
		//
		$private_pages = $this->db->execute("SELECT {pages}.*, {pages_users_link}.user_id,{users}.handle " .
									" FROM {pages},{pages_users_link} " .
									" LEFT JOIN {users} ON {users}.user_id = {pages_users_link}.user_id ".
									" WHERE {pages}.default_entry_flag = "._OFF." AND {pages}.private_flag = "._ON.
									" AND {pages}.thread_num = 0 AND {pages}.room_id = {pages_users_link}.room_id AND ({pages}.insert_user_id != {pages_users_link}.user_id OR {pages}.insert_user_name != {users}.handle) ");
		if ($private_pages !== false && isset($private_pages[0])) {
			foreach($private_pages as $private_page) {
				if(!isset($private_page['handle'])) {
					$private_page['handle'] = "";
				}
				$where_params = array(
									"(page_id=".$private_page['page_id']." OR root_id=".$private_page['page_id'].")" => null
								);
				
				$this->db->updateExecute("pages", array("insert_user_id" => $private_page['user_id'], "insert_user_name" => $private_page['handle']), $where_params);
			}
		}
		
		// 
		// pagesにカラム追加
		//
		$metaColumns = $adodb->MetaColumns($this->db->getPrefix()."pages");
		if(!isset($metaColumns["PERMALINK"])) {
			//set_time_limit(2400);
			$sql = "ALTER TABLE `".$this->db->getPrefix()."pages` 
						ADD `permalink` TEXT NOT NULL AFTER `page_name` ;";
			$result = $this->db->execute($sql);
			if($result === false) return false;
			
			$sql = "ALTER TABLE `".$this->db->getPrefix()."pages` 
						ADD FULLTEXT (`permalink`);";
			$result = $this->db->execute($sql);
			if($result === false) return false;

			//
			// permalinkセット
			//
			$pages = $this->db->execute("SELECT {users}.handle, {pages}.* " .
										" FROM {pages}" .
										" LEFT JOIN {users} ON {users}.user_id = {pages}.insert_user_id".
										" ORDER BY {pages}.thread_num,{pages}.display_sequence");
			if($pages === false) {
				return false;
			}
			if(is_array($pages)) {
				$set_permalink_arr = array();
				$set_parent_id_arr = array();
				foreach($pages as $page) {
					if(($page['space_type'] == _SPACE_TYPE_PUBLIC && $page['thread_num'] == 1 && $page['display_sequence'] == 1) ||
						$page['display_sequence'] == 0) {
						// パブリックスペース OR トップページ　OR　グループスペース
						$set_parent_id_arr[$page['page_id']] = null;
						$set_permalink_arr[$page['page_id']] = "";
						continue;
					} else if($page['private_flag'] == _ON && $page['thread_num'] == 0) {
						// マイポータル、マイルーム直下
						// handle名セット
						if(isset($page['user_id'])) {
							$page['page_name'] = $page['handle'];
						} else {
							$page['page_name'] = $page['insert_user_name'];
						}
					}
					$permalink = "";
					$set_parent_id_arr[$page['page_id']] = $page['parent_id'];
					if(isset($set_permalink_arr[$page['parent_id']]) && $set_permalink_arr[$page['parent_id']] != "") {
						// 親のpermalinkあり
						$permalink = $set_permalink_arr[$page['parent_id']];
					} else {
						if($page['space_type'] == _SPACE_TYPE_PUBLIC && _PERMALINK_PUBLIC_PREFIX_NAME != '') {
							$permalink = _PERMALINK_PUBLIC_PREFIX_NAME;
						} else if($page['space_type'] == _SPACE_TYPE_GROUP && $page['private_flag'] == _ON &&
							$page['default_entry_flag'] == _ON && _PERMALINK_MYPORTAL_PREFIX_NAME != '') {
							$permalink = _PERMALINK_MYPORTAL_PREFIX_NAME;
						} else if($page['space_type'] == _SPACE_TYPE_GROUP && $page['private_flag'] == _ON &&
							$page['default_entry_flag'] == _OFF && _PERMALINK_PRIVATE_PREFIX_NAME != '') {
							$permalink = _PERMALINK_PRIVATE_PREFIX_NAME;
						} else if($page['space_type'] == _SPACE_TYPE_GROUP && $page['private_flag'] == _OFF &&
							_PERMALINK_GROUP_PREFIX_NAME != '') {
							$permalink = _PERMALINK_GROUP_PREFIX_NAME;
						}
					}
					
					$parent_permalink = $permalink;
					//if(substr($parent_permalink, -1, 1) == '/') {
					//	$parent_permalink = substr($parent_permalink, 0, strlen($parent_permalink) - 1);
					//}
					if(($page['space_type'] == _SPACE_TYPE_PUBLIC && $page['thread_num'] == 0) ||
						($page['private_flag'] == _OFF && $page['space_type'] == _SPACE_TYPE_GROUP && $page['thread_num'] == 0)) {
						$replace_permalink = "";			
					} else {
						$replace_permalink = preg_replace(_PERMALINK_PROHIBITION, _PERMALINK_PROHIBITION_REPLACE, $page['page_name']);
						
						if($permalink == "") {
							$permalink .= $replace_permalink;
						} else {
							$permalink .= '/' . $replace_permalink;	
						}
					}
					
					// 同じ階層に同じ名称の固定リンクがあればリネーム
					if($permalink !== "") {
						$parent_permalink = ($parent_permalink != "") ? $parent_permalink.'/' : $parent_permalink;
						$count = 1;
						if(preg_match(_PERMALINK_PROHIBITION_DIR_PATTERN, basename($permalink))) {
							$permalink = $parent_permalink. $replace_permalink ."-". $count;
							$count++;
						}
						
						while(1) {
							$same_pages = $this->db->selectExecute("pages", array("permalink"=> $permalink), null, 1);
							if(isset($same_pages[0])) {
								$permalink = $parent_permalink. $replace_permalink ."-". $count;
							} else {
								break;
							}
							$count++;
						}
					}
					
					$set_permalink_arr[$page['page_id']] = $permalink;
					
					if(isset($permalink)) {
						$params = array(
				    		"permalink" => $permalink
						);
						
						$where_params = array(
				    		"page_id" => $page['page_id']
						);
						$result = $this->db->updateExecute("pages", $params, $where_params, false);
						if($result === false) {
							return false;
						}	
					}
				}
			}
			
			// pagesテーブルにカラムを追加するバージョンで同時にtextarea_styleのバグを修正
			$this->db->execute("UPDATE `".$this->db->getPrefix()."textarea_style` SET `css` = 'border-right-color' WHERE css = 'boder-right-color' LIMIT 1");
		}
		
		//
		// pages_meta_inf追加
		//
		$adodb = $this->db->getAdoDbObject();
		$metaTables = $adodb->MetaTables();
		if (!in_array($this->db->getPrefix()."pages_meta_inf", $metaTables)) {
			$sql = "CREATE TABLE `".$this->db->getPrefix()."pages_meta_inf` (" .
					"`page_id`             int(11) UNSIGNED NOT NULL,".
					"`title`               varchar(255),".
					"`meta_keywords`       text,".
					"`meta_description`    text,".
					"`insert_time`         varchar(14) NOT NULL default '',".
					"`insert_site_id`      varchar(40) NOT NULL default '',".
					"`insert_user_id`      varchar(40) NOT NULL default '',".
					"`insert_user_name`    varchar(255) NOT NULL default '',".
					"`update_time`         varchar(14) NOT NULL default '',".
					"`update_site_id`      varchar(40) NOT NULL default '',".
					"`update_user_id`      varchar(40) NOT NULL default '',".
					"`update_user_name`    varchar(255) NOT NULL default '',".
					" PRIMARY KEY (`page_id`)" .
					") TYPE=MyISAM;";
			$result = $this->db->execute($sql);
			if ($result === false) {
				return false;
			}
		}
		
		return true;
	}
}
?>
